# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
import json
import unittest
import requests

from superdesk.publish import SUBSCRIBER_TYPES
from superdesk.publish.transmitters.http_push import HTTPPushService

from unittest import mock
from unittest.mock import Mock
from superdesk.errors import PublishHTTPPushServerError, PublishHTTPPushClientError


def get_fixture(fixture):
    filename = os.path.join(os.path.dirname(os.path.realpath(__file__)), '%s.json' % fixture)
    with open(filename, 'r') as file:
        return json.load(file)


class ItemNotFound(Exception):
    pass


class NotFoundResponse():
    status_code = 404


class CreatedResponse():
    status_code = 201


class HTTPPushPublishTestCase(unittest.TestCase):

    def setUp(self):
        if 'HTTP_PUSH_RESOURCE_URL' not in os.environ:
            self.resource_url = ''
        else:
            self.resource_url = os.environ['HTTP_PUSH_RESOURCE_URL']

        self.subscribers = [{
            "_id": "1", "name": "Test", "media_type": "media",
            "subscriber_type": SUBSCRIBER_TYPES.DIGITAL, "is_active": True,
            "sequence_num_settings": {"max": 10, "min": 1},
            "destinations": [{
                "name": "test", "delivery_type": "http_push", "format": "ninjs",
                "config": {"resource_url": self.resource_url, "secret_token": "123456789"}
            }]
        }]
        self.formatted_item1 = {"_id": "item1",
                                "headline": "headline",
                                "versioncreated": "2015-03-09T16:32:23",
                                "version": 1
                                }
        self.formatted_item2 = {"_id": "item1",
                                "headline": "headline2",
                                "versioncreated": "2015-03-21T13:43:51",
                                "version": 2
                                }
        self.item = {'item_id': 'item1',
                     'format': 'ninjs',
                     'item_version': 1,
                     'published_seq_num': 1,
                     'formatted_item': json.dumps(self.formatted_item1),
                     'destination': {"name": "test", "delivery_type": "http_push", "format": "ninjs",
                                     "config": {"resource_url": self.resource_url, "secret_token": "123456789"}
                                     }}

        self.destination = self.item.get('destination', {})

    def is_item_published(self, item_id):
        """Return True if the item was published, False otherwise.

        Raises Exception in case of server/communication error.
        """
        if not getattr(self, 'resource_url', None):
            return

        response = requests.get(self.getItemURL(item_id))
        if response.status_code == requests.codes.not_found:  # @UndefinedVariable
            return False
        self.assertEqual(response.status_code, requests.codes.ok,  # @UndefinedVariable
                         'Error retrieving item from the content API')
        return True

    def getItemURL(self, item_id):
        """Returns the URL for item read

        @param item_id: the item identifier
        @return: string
        """
        return '%s/%s' % (self.resource_url, item_id)

    def test_get_assets_url(self):
        service = HTTPPushService()
        self.assertEqual(service._get_assets_url(self.destination), None)

    def test_get_resource_url(self):
        service = HTTPPushService()
        self.assertEqual(service._get_resource_url(self.destination), self.resource_url)

    def test_get_secret_token(self):
        service = HTTPPushService()
        self.assertEqual(service._get_secret_token(self.destination), "123456789")

    def test_get_headers(self):
        service = HTTPPushService()
        headers = service._get_headers('test payload', self.destination, {})
        self.assertEqual('sha1=8be62a607898504f87559cb52dc23f9ebee65a21', headers[service.hash_header])

    def test_publish_an_item(self):
        if not getattr(self, 'resource_url', None):
            return

        service = HTTPPushService()

        service._transmit(self.item, self.subscribers)
        self.assertTrue(self.is_item_published(self.item['item_id']))

        self.item['formatted_item'] = json.dumps(self.formatted_item2)
        service._transmit(self.item, self.subscribers)
        item = requests.get(self.getItemURL(self.item['item_id'])).json()
        self.assertEqual(item['headline'], 'headline2')
        self.assertEqual(item['version'], 2)

    @mock.patch('superdesk.errors.notifiers')
    @mock.patch('requests.post')
    def test_client_publish_error_thrown(self, fake_post, fake_notifiers):
        raise_http_exception = Mock(side_effect=PublishHTTPPushClientError.httpPushError(Exception('client 4xx')))

        fake_post.return_value = Mock(status_code=401, text='client 4xx', raise_for_status=raise_http_exception)

        # needed for bad exception handling classes
        fake_notifiers.return_value = []

        service = HTTPPushService()

        with self.assertRaises(PublishHTTPPushClientError):
            service._push_item(self.destination, json.dumps(self.item))

    @mock.patch('superdesk.errors.notifiers')
    @mock.patch('requests.post')
    def test_server_publish_error_thrown(self, fake_post, fake_notifiers):
        raise_http_exception = Mock(side_effect=PublishHTTPPushServerError.httpPushError(Exception('server 5xx')))

        fake_post.return_value = Mock(status_code=503, text='server 5xx', raise_for_status=raise_http_exception)

        # needed for bad exception handling classes
        fake_notifiers.return_value = []

        service = HTTPPushService()

        with self.assertRaises(PublishHTTPPushServerError):
            service._push_item(self.destination, json.dumps(self.item))

    @mock.patch('superdesk.publish.transmitters.http_push.app')
    @mock.patch('requests.post', return_value=CreatedResponse)
    @mock.patch('requests.get', return_value=NotFoundResponse)
    def test_push_associated_assets(self, get_mock, post_mock, app_mock):
        app_mock.media.get.return_value = 'bin'

        dest = {'config': {'assets_url': 'http://example.com'}}
        item = get_fixture('package')

        service = HTTPPushService()
        service._copy_published_media_files({}, dest)

        get_mock.assert_not_called()
        post_mock.assert_not_called()

        service._copy_published_media_files(item, dest)

        images = [
            # embedded original
            '2017020111028/9a836848c3c3387a151dbed96e83b7d50e6b0e71ca397e0b1dc0f4b2f4127acd.jpg',
            # main-0 original
            '20170201110216/d3ad29bafe0710c42b7cfc201939f266c6ca5c11a713625388decff4da87ba5b.jpg',
            # embedded thumbnail
            '2017020111028/a0502320d6d07dd921253171e971943adf791eb2b34dfe82da73c053a343a7c2.jpg',
        ]

        for media in images:
            get_mock.assert_any_call('http://example.com/%s' % media)
            post_mock.assert_any_call('http://example.com',
                                      files={'media': (media, app_mock.media.get.return_value, 'image/jpeg', {})},
                                      data={'media_id': media})

    @mock.patch('superdesk.publish.transmitters.http_push.app')
    @mock.patch('requests.post', return_value=CreatedResponse)
    @mock.patch('requests.get', return_value=NotFoundResponse)
    def test_push_attachments(self, get_mock, post_mock, app_mock):
        app_mock.media.get.return_value = 'bin'

        dest = {'config': {'assets_url': 'http://example.com'}}
        item = {
            'type': 'text',
            'attachments': [
                {'id': 'foo', 'media': 'media-id', 'mimetype': 'text/plain'},
            ]
        }

        service = HTTPPushService()
        service._copy_published_media_files(item, dest)

        app_mock.media.get.assert_called_with('media-id', resource='attachments')
        get_mock.assert_called_with('http://example.com/media-id')
        post_mock.assert_called_with(
            'http://example.com',
            files={'media': ('media-id', app_mock.media.get.return_value, 'text/plain', {})},
            data={'media_id': 'media-id'})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from abc import ABCMeta, abstractmethod

from superdesk.etree import etree
from superdesk.errors import SkipValue
from flask import current_app as app
from superdesk.metadata.item import Priority
from collections import OrderedDict
import superdesk
import logging

logger = logging.getLogger(__name__)


class FeedParser(metaclass=ABCMeta):
    """
    Base class for a Feed Parser.

    A Feed Parser class must have the following attributes:
        1. `NAME` - unique name under which to register the class.
    """

    @abstractmethod
    def can_parse(self, article):
        """Sub-classes must override this method and tell whether it can parse the given article.

        :param article: article
        :return: True if the feed parser can parse, False otherwise.
        :rtype: bool
        """
        raise NotImplementedError()

    @abstractmethod
    def parse(self, article, provider=None):
        """Parse the given article and extracts the relevant elements/attributes values from the given article.

        :param article: XML String to parse
        :type article: str
        :param provider: Ingest Provider Details, defaults to None
        :type provider: dict having properties defined in
                        :py:class: `superdesk.io.ingest_provider_model.IngestProviderResource`
        :return: parsed data as dict.
        :rtype: dict having properties defined in :py:mod: `superdesk.metadata.item`
        """
        raise NotImplementedError()

    def set_dateline(self, item, city=None, text=None):
        """Sets the 'dateline' to the article identified by item.

        If city is passed then the system checks if city is available in Cities collection.
        If city is not found in Cities collection then dateline's located is set with default values.

        :param item: article.
        :type item: dict
        :param city: Name of the city, if passed the system will search in Cities collection.
        :type city: str
        :param text: dateline in full. For example, "STOCKHOLM, Aug 29, 2014"
        :type text: str
        """

        item.setdefault('dateline', {})

        if city:
            cities = app.locators.find_cities()
            located = [c for c in cities if c['city'] == city]
            item['dateline']['located'] = located[0] if len(located) > 0 else {'city_code': city, 'city': city,
                                                                               'tz': 'UTC', 'dateline': 'city'}
        if text:
            item['dateline']['text'] = text

    def map_priority(self, source_priority):
        """
        Maps the source priority to superdesk priority

        :param source_priority:
        :type source_priority: str

        :return: priority of the item
        :rtype int
        """

        if source_priority and source_priority.isdigit():
            if int(source_priority) in Priority.values():
                return int(source_priority)

        return Priority.Ordinary.value


class XMLFeedParser(FeedParser, metaclass=ABCMeta):
    """
    Base class for Feed Parsers which can parse the XML Content.
    """

    def __init__(self):
        self.root = None
        self.metadata_mapping = None

    def _parse_mapping(self, value):
        if isinstance(value, dict):
            if 'default_attr' in value:
                if 'default' in value:
                    logger.error("default and default_attr can't be used at the same time,"
                                 "only default will be used ({})".format(self.__class__))
                if 'xpath':
                    if '/' not in 'xpath':
                        logger.error("default_attr can be used for simple child element ({})".format(self.__class__))
                else:
                    logger.error("xpath is needed when default_attr is used ({})".format(self.__class__))
            if 'callback' in value and 'list' in value:
                del value['list']
                logger.error("list can't ve used with callback ({})".format(self.__class__))
            return value
        elif isinstance(value, str):
            if not value:
                return {}
            return {'xpath': value}
        elif callable(value):
            return {'callback': value}
        else:
            logger.warn("Can't parse mapping value {}, ignoring it".format(value))
            return {}

    def _generate_mapping(self, setting_param_name):
        """Generate self.metadata_mapping according to available mappings.

        The following mappings are used in this order (last is more important):
            - self.default_mapping
            - self.MAPPING, intended for subclasses
            - [setting_param_name] dictionary which can be put in settings
        If a value is a non-empty string, it is a xpath, @attribute can be used as last path component.
        If value is empty string/dict, the key will be ignored
        If value is a callable, it will be executed with nitf Element as argument, return value will be used.
        If a dictionary is used as value, following keys can be used:
            xpath: path to the element
            callback: callback executed with nitf Element as argument, return value will be used
            default: value to use if element/attribute doesn't exists (default: doesn't set the key)
            list: a bool which indicate if a list is expected
                  if False (default), only first value is used
            filter: callable to be used with found element/value
            default_attr: value if element exist but attribute is missing
                this works actually for all values, if it is not found parent element is checked
                and default_attr is used only if parent element exists
            key_hook: a callable which store itself the resulting value in the item,
                      usefull for specific behaviours when several values goes to same key
                      callable will get item and value as arguments.
            update: a bool which indicate that default mapping must be updated instead of overwritten
        Note the difference between using a callable directly, and "filter" in a dict:
        the former get the root element and can be skipped with SkipValue, while the
        later get an element/value found with xpath.
        """
        try:
            class_mapping = self.MAPPING
        except AttributeError:
            class_mapping = {}

        if setting_param_name is not None:
            settings_mapping = getattr(superdesk.config, setting_param_name)
            if settings_mapping is None:
                logging.info("No mapping found in settings for NITF parser, using default one")
                settings_mapping = {}
        else:
            settings_mapping = {}

        mapping = self.metadata_mapping = OrderedDict()

        for source_mapping in (self.default_mapping, class_mapping, settings_mapping):
            for key, value in source_mapping.items():
                key_mapping = self._parse_mapping(value)
                if key_mapping.get('update', False) and key in mapping:
                    mapping[key].update(key_mapping)
                else:
                    mapping[key] = key_mapping

    def do_mapping(self, item, item_xml, setting_param_name=None, namespaces=None):
        """Apply mapping to item's XML content to get article metadata

        mapping is generated by self._generate_mapping
        :param item: dictionary to fill with item metadata
        :type item: dict
        :param item_xml: XML element to parse
        :type item_xml: lxml.etree.Element
        :param setting_param_name: name of the settings attribute containing the mapping
        :type setting_param_name: str
        :type setting_param_name: NoneType
        :param namespaces: namespaces map to use with lxml methods
        :type namespaces: dict
        :type namespaces: NoneType
        """
        if self.metadata_mapping is None:
            self._generate_mapping(setting_param_name)

        for key, mapping in self.metadata_mapping.items():
            if not mapping:
                # key is ignored
                continue
            try:
                xpath = mapping['xpath']
            except KeyError:
                # no xpath, we must have a callable
                try:
                    values = [mapping['callback'](item_xml)]
                except KeyError:
                    logging.warn("invalid mapping for key {}, ignoring it".format(key))
                    continue
                except SkipValue:
                    continue
                list_ = False
            else:
                values = item_xml.xpath(xpath, namespaces=namespaces)
                list_ = mapping.get('list', False)
                if not list_:
                    values = values[:1]
                if not values:
                    # nothing found, we check default
                    try:
                        values = [mapping['default']]
                    except KeyError:
                        if 'default_attr' in mapping:
                            parent = item_xml.xpath(xpath[:xpath.rfind('/')], namespaces=namespaces)
                            if parent:
                                # default_attr is only used when there is a parent element
                                values = [mapping['default_attr']]
                            else:
                                continue
                        else:
                            # if there is not default value we skip the key
                            continue
                else:
                    for idx, current_value in enumerate(values):
                        if isinstance(current_value, str):
                            if 'filter' in mapping:
                                values[idx] = mapping['filter'](current_value)
                        else:
                            # we have an element,
                            # do we want a filter or the content?
                            try:
                                # filter
                                values[idx] = mapping['filter'](current_value)
                            except KeyError:
                                # content
                                values[idx] = ''.join(current_value.itertext())

            value = values if list_ else values[0]
            if 'key_hook' in mapping:
                mapping['key_hook'](item, value)
            else:
                item[key] = value

        return item

    def qname(self, tag, ns=None):
        """
        Return the Qualified Name of given XML tag.

        :param tag: XML Tag
        :type tag: str
        :param ns: Namespace to be used for generating Qualified Name, defaults to None.
        :type ns: str
        :return: Qualified Name of tag
        :rtype: str
        """

        if ns is None:
            ns = self.root.tag.rsplit('}')[0].lstrip('{')
        elif ns is not None and ns == 'xml':
            ns = 'http://www.w3.org/XML/1998/namespace'

        return str(etree.QName(ns, tag))


class FileFeedParser(FeedParser, metaclass=ABCMeta):
    """
    Base class for Feed Parsers which can parse the content in a file.
    """

    pass


class EmailFeedParser(FeedParser, metaclass=ABCMeta):
    """
    Base class for Feed Parsers which can parse email message.
    """

    pass


# must be imported for registration
from superdesk.io.feed_parsers.anpa import ANPAFeedParser  # NOQA
from superdesk.io.feed_parsers.iptc7901 import IPTC7901FeedParser  # NOQA
from superdesk.io.feed_parsers.newsml_1_2 import NewsMLOneFeedParser  # NOQA
from superdesk.io.feed_parsers.newsml_2_0 import NewsMLTwoFeedParser  # NOQA
from superdesk.io.feed_parsers.nitf import NITFFeedParser  # NOQA
from superdesk.io.feed_parsers.rfc822 import EMailRFC822FeedParser  # NOQA
from superdesk.io.feed_parsers.wenn_parser import WENNFeedParser  # NOQA
from superdesk.io.feed_parsers.dpa_iptc7901 import DPAIPTC7901FeedParser  # NOQA
from superdesk.io.feed_parsers.afp_newsml_1_2 import AFPNewsMLOneFeedParser  # NOQA
from superdesk.io.feed_parsers.scoop_newsml_2_0 import ScoopNewsMLTwoFeedParser  # NOQA
from superdesk.io.feed_parsers.ap_anpa import AP_ANPAFeedParser  # NOQA
from superdesk.io.feed_parsers.pa_nitf import PAFeedParser  # NOQA
from superdesk.io.feed_parsers.efe_nitf import EFEFeedParser  # NOQA
from superdesk.io.feed_parsers.wordpress_wxr import WPWXRFeedParser  # NOQA
from superdesk.io.feed_parsers.ninjs import NINJSFeedParser  # NOQA


# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask import Flask
from unittest import mock

from content_api.tests import ApiTestCase


class PackagesServiceTestCase(ApiTestCase):
    """Base class for the `packages` service tests."""

    def _get_target_class(self):
        """Return the class under test.

        Make the test fail immediately if the class cannot be imported.
        """
        try:
            from content_api.packages import PackagesService
        except ImportError:
            self.fail("Could not import class under test (PackagesService).")
        else:
            return PackagesService

    def _make_one(self, *args, **kwargs):
        """Create a new instance of the class under test."""
        return self._get_target_class()(*args, **kwargs)


@mock.patch("content_api.packages.service.ItemsService.on_fetched_item")
class OnFetchedItemMethodTestCase(PackagesServiceTestCase):
    """Tests for the on_fetched_item() method."""

    def setUp(self):
        super().setUp()

        self.app = Flask(__name__)
        self.app.config['CONTENTAPI_URL'] = 'http://content_api.com'
        self.app.config['URLS'] = {
            'items': 'items_endpoint',
            'packages': 'packages_endpoint'
        }

        self.app_context = self.app.app_context()
        self.app_context.push()

    def tearDown(self):
        self.app_context.pop()
        super().tearDown()

    def test_invokes_superclass_method_with_correct_args(self, super_fetched):
        document = {'_id': 'item:XYZ'}
        instance = self._make_one(datasource='packages')
        instance.on_fetched_item(document)
        super_fetched.assert_called_with(document)

    def test_sets_uri_field_on_referenced_items(self, super_fetched):
        document = {
            '_id': 'item:XYZ',
            'associations': {
                'main': {'type': 'picture', '_id': 'img:123'}
            }
        }

        instance = self._make_one(datasource='packages')
        instance.on_fetched_item(document)

        expected_assoc = {
            'main': {
                'type': 'picture',
                'uri': 'http://content_api.com/items_endpoint/img%3A123'
            }
        }
        self.assertEqual(document.get('associations'), expected_assoc)

    def test_sets_uri_field_on_referenced_packages(self, super_fetched):
        document = {
            '_id': 'item:XYZ',
            'associations': {
                'story_object': {'type': 'composite', '_id': 'pkg:456'}
            }
        }

        instance = self._make_one(datasource='packages')
        instance.on_fetched_item(document)

        expected_assoc = {
            'story_object': {
                'type': 'composite',
                'uri': 'http://content_api.com/packages_endpoint/pkg%3A456'
            }
        }
        self.assertEqual(document.get('associations'), expected_assoc)


@mock.patch("content_api.packages.service.ItemsService.on_fetched")
class OnFetchedMethodTestCase(PackagesServiceTestCase):
    """Tests for the on_fetched() method."""

    def setUp(self):
        super().setUp()

        self.app = Flask(__name__)
        self.app.config['CONTENTAPI_URL'] = 'http://content_api.com'
        self.app.config['URLS'] = {
            'items': 'items_endpoint',
            'packages': 'packages_endpoint'
        }

        self.app_context = self.app.app_context()
        self.app_context.push()

    def tearDown(self):
        self.app_context.pop()
        super().tearDown()

    def test_invokes_superclass_method_with_correct_args(self, super_fetched):
        result = {
            '_items': []
        }
        instance = self._make_one(datasource='packages')
        instance.on_fetched(result)
        super_fetched.assert_called_with(result)

    def test_sets_uri_field_on_objects_referenced_by_fetched_packages(
        self, super_fetched
    ):
        result = {
            '_items': [
                {
                    '_id': 'pkg:ABC',
                    'associations': {
                        'main_picture': {'type': 'picture', '_id': 'img:123'}
                    }
                },
                {
                    '_id': 'pkg:DEF',
                    'associations': {
                        'main_story': {'type': 'composite', '_id': 'pkg:456'}
                    }
                },
            ]
        }

        instance = self._make_one(datasource='packages')
        instance.on_fetched(result)

        # check 1st fetched package's associations
        expected_assoc = {
            'main_picture': {
                'type': 'picture',
                'uri': 'http://content_api.com/items_endpoint/img%3A123'
            }
        }
        fetched_package = result['_items'][0]
        self.assertEqual(fetched_package.get('associations'), expected_assoc)

        # check 2nd fetched package's associations
        expected_assoc = {
            'main_story': {
                'type': 'composite',
                'uri': 'http://content_api.com/packages_endpoint/pkg%3A456'
            }
        }
        fetched_package = result['_items'][1]
        self.assertEqual(fetched_package.get('associations'), expected_assoc)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from unittest import mock


from superdesk.tests import TestCase
from superdesk.publish.formatters.newsml_g2_formatter import NewsMLG2Formatter
from superdesk.publish import init_app
from superdesk.publish.formatters import Formatter
from superdesk.publish.subscribers import SubscribersService
from lxml import etree
import datetime

NOW = datetime.datetime.now(datetime.timezone.utc)
ARTICLE = {
    '_id': 'urn:localhost.abc',
    'guid': 'urn:localhost.abc',
    'versioncreated': NOW,
    'firstcreated': NOW,
    'source': 'AAP',
    'anpa_category': [{'qcode': 'a', 'name': 'Australian General News'}],
    'headline': 'This is a test headline',
    'byline': 'joe',
    'slugline': 'slugline',
    'subject': [{'qcode': '02011001', 'name':'test'}, {'qcode': '02011002', 'name':'test'}],
    'anpa_take_key': 'take_key',
    'unique_id': '1',
    'body_html': '<p><h1>The story body</h1><h3/>empty element on purpose<br/><strong>test</strong><em/><br/>'
                 'other test</p>',
    'type': 'text',
    'word_count': '1',
    'priority': 1,
    '_current_version': 5,
    'state': 'published',
    'urgency': 2,
    'pubstatus': 'usable',
    'dateline': {
        'source': 'AAP',
        'text': 'sample dateline',
        'located': {
            'alt_name': '',
            'state': 'California',
            'city_code': 'Los Angeles',
            'city': 'Los Angeles',
            'dateline': 'city',
            'country_code': 'US',
            'country': 'USA',
            'tz': 'America/Los_Angeles',
            'state_code': 'CA'
        }
    },
    'keywords': ['traffic'],
    'abstract': 'sample abstract',
    'place': [
        {'qcode': 'NSW', 'name': 'NSW', 'state': 'New South Wales',
         'country': 'Australia', 'world_region': 'Oceania'}
    ],
    'ednote': '',
    'body_footer': '',
    'company_codes': [{'name': 'YANCOAL AUSTRALIA LIMITED', 'qcode': 'YAL', 'security_exchange': 'ASX'}]
}


class NewsMLG2FormatterTest(TestCase):
    article = None

    @mock.patch.object(SubscribersService, 'generate_sequence_number', lambda self, subscriber: 1)
    def setUp(self):
        self.formatter = NewsMLG2Formatter()
        self.base_formatter = Formatter()
        init_app(self.app)
        if self.article is None:
            self.article = ARTICLE
            # formatting is done once for all tests to save time
            # as long as used attributes are not modified, it's fine
            self.formatter_output = self.formatter.format(self.article, {'name': 'Test NewsMLG2'})
            self.doc = self.formatter_output[0][1]
            self.doc_xml = etree.fromstring(self.doc.encode('utf-8'))

    def test_html_void(self):
        import ipdb
        ipdb.set_trace()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from nose.tools import assert_raises

from apps.archive.common import get_item_expiry
from apps.tasks import (
    apply_stage_rule, apply_onstage_rule, compare_dictionaries,
    MACRO_INCOMING, MACRO_OUTGOING, MACRO_ONSTAGE
)
from superdesk.errors import SuperdeskApiError
from superdesk.utc import get_expiry_date
from superdesk.tests import TestCase


class TasksTestCase(TestCase):

    def test_get_global_content_expiry(self):
        calculated_minutes = get_item_expiry(desk=None, stage=None)
        reference_minutes = get_expiry_date(self.ctx.app.config['CONTENT_EXPIRY_MINUTES'])
        self.assertEquals(calculated_minutes.hour, reference_minutes.hour)
        self.assertEquals(calculated_minutes.minute, reference_minutes.minute)

    def test_get_desk_content_expiry(self):
        desk = {"content_expiry": 10}
        calculated_minutes = get_item_expiry(desk=desk, stage=None)
        reference_minutes = get_expiry_date(10)
        self.assertEquals(calculated_minutes.hour, reference_minutes.hour)
        self.assertEquals(calculated_minutes.minute, reference_minutes.minute)

    def test_get_stage_content_expiry(self):
        stage = {"content_expiry": 10}
        calculated_minutes = get_item_expiry(desk=None, stage=stage)
        reference_minutes = get_expiry_date(10)
        self.assertEquals(calculated_minutes.hour, reference_minutes.hour)
        self.assertEquals(calculated_minutes.minute, reference_minutes.minute)

    def test_apply_incoming_stage_rule(self):
        doc = {'id': '1', 'body_html': 'Test-1'}
        update = {'anpa_take_key': 'x'}
        stage = {'incoming_macro': 'populate_abstract'}
        with self.app.app_context():
            apply_stage_rule(doc, update, stage, MACRO_INCOMING)
            self.assertEquals(update['abstract'], 'Test-1')

    def test_apply_outgoing_stage_rule(self):
        doc = {'id': '1', 'body_html': 'Test-1'}
        update = {'anpa_take_key': 'x'}
        stage = {'outgoing_macro': 'populate_abstract'}
        with self.app.app_context():
            apply_stage_rule(doc, update, stage, MACRO_OUTGOING)
            self.assertEquals(update['abstract'], 'Test-1')

    def test_apply_stage_incoming_validation_rule(self):
        doc = {'id': '1', 'body_html': 'Test-1'}
        update = {'headline': 'x'}
        stage = {'incoming_macro': 'take_key_validator'}
        with self.app.app_context():
            with assert_raises(SuperdeskApiError):
                apply_stage_rule(doc, update, stage, MACRO_INCOMING)

    def test_apply_stage_incoming_validation_rule_passes(self):
        doc = {'id': '1', 'body_html': 'Test-1', 'anpa_take_key': 'a'}
        update = {'headline': 'x'}
        stage = {'incoming_macro': 'take_key_validator'}
        with self.app.app_context():
            apply_stage_rule(doc, update, stage, MACRO_INCOMING)

    def test_apply_stage_incoming_validation_rule_ignored(self):
        doc = {'id': '1', 'body_html': 'Test-1'}
        update = {'headline': 'x'}
        stage = {'outgoing_macro': 'take_key_validator'}
        with self.app.app_context():
            apply_stage_rule(doc, update, stage, MACRO_INCOMING)

    def test_apply_stage_outgoing_validation_rule_ignored(self):
        doc = {'id': '1', 'body_html': 'Test-1'}
        update = {'headline': 'x'}
        stage = {'incoming_macro': 'take_key_validator'}
        with self.app.app_context():
            apply_stage_rule(doc, update, stage, MACRO_OUTGOING)

    def test_apply_stage_outgoing_validation_rule(self):
        doc = {'id': '1', 'body_html': 'Test-1'}
        update = {'headline': 'x'}
        stage = {'outgoing_macro': 'take_key_validator'}
        with self.app.app_context():
            with assert_raises(SuperdeskApiError):
                apply_stage_rule(doc, update, stage, MACRO_OUTGOING)

    def test_apply_on_stage_validation_rule(self):
        doc = {'id': '1', 'body_html': 'Test-1'}
        update = {'headline': 'x'}
        stage = {'onstage_macro': 'take_key_validator'}
        with self.app.app_context():
            with assert_raises(SuperdeskApiError):
                apply_stage_rule(doc, update, stage, MACRO_ONSTAGE)

    def test_apply_onstage_rule(self):
        doc = {'id': '1', 'body_html': 'Test-1', 'task': {'stage': 1}}
        stages = [{'_id': 1, 'onstage_macro': 'take_key_validator'}]
        self.app.data.insert('stages', stages)

        with self.app.app_context():
            with assert_raises(SuperdeskApiError):
                apply_onstage_rule(doc, 1)

    def test_apply_onstage_rule_applies(self):
        doc = {'id': '1', 'body_html': 'Test-1', 'task': {'stage': 1}}
        stages = [{'_id': 1, 'onstage_macro': 'populate_abstract'}]
        self.app.data.insert('stages', stages)

        with self.app.app_context():
            apply_onstage_rule(doc, 1)
            self.assertEquals(doc['abstract'], 'Test-1')

    def test_compare_dictionaries(self):
        original = {
            'id': 1,
            'body_html': 'Test-1'
        }

        updates = {
            'body_html': 'Test-2',
            'headline': 'a',
        }

        modified = compare_dictionaries(original, updates)
        self.assertEquals(2, len(modified))
        self.assertTrue('body_html' in modified)
        self.assertTrue('headline' in modified)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from datetime import datetime, timedelta

from flask import current_app as app

import superdesk


class DictObject:
    def __init__(self, **args):
        self.__dict__.update(args)


class AuthDataManager:

    @staticmethod
    def get_client(client_id):
        """Loads a client from the database and returns it as an object or None.
        """
        client = superdesk.get_resource_service('clients').find_one(req=None, _id=client_id)
        if not client:
            return None
        client['client_id'] = client['_id']
        client['default_scopes'] = app.config['OAUTH2_SCOPES']
        client['default_redirect_uri'] = app.config['CONTENTAPI_URL']
        return DictObject(**client)

    @staticmethod
    def get_user(username, password, *args, **kwargs):
        """Loads a user from the database and returns it as an object or None.
        """
        user_service = superdesk.get_resource_service('users')
        user = user_service.find_one(req=None, username=username) or {}
        user_password = user.get('password', '').encode('UTF-8')
        user = user if user_service.password_match(password.encode('UTF-8'), user_password) else {}
        user = DictObject(**user)
        return user if getattr(user, 'username', None) else None

    @staticmethod
    def get_token(access_token=None, refresh_token=None):
        """Loads a token from the database and returns it as an object or None.
        """
        if not (access_token or refresh_token):
            return None

        if access_token:
            token = superdesk.get_resource_service('tokens').find_one(req=None, access_token=access_token)
        elif refresh_token:
            token = superdesk.get_resource_service('tokens').find_one(req=None, refresh_token=refresh_token)
        return DictObject(token)

    @staticmethod
    def save_token(token, request, *args, **kwargs):
        """Saves a token to the database
        """
        client_id = request.client.client_id
        user_id = request.user._id

        # Make sure there is only one grant token for every (client, user)
        old_tokens = superdesk.get_resource_service('tokens').get(req=None,
                                                                  lookup={'client': client_id, 'user': user_id})
        for old_token in old_tokens:
            app.redis.delete(old_token['access_token'])
        superdesk.get_resource_service('tokens').delete({'client': client_id, 'user': user_id})

        expires_in = token.pop('expires_in')
        expires = datetime.utcnow() + timedelta(seconds=expires_in)

        token = {
            'client': request.client.client_id,
            'user': user_id,
            'token_type': token['token_type'],
            'access_token': token['access_token'],
            'refresh_token': token['refresh_token'],
            'expires': expires
        }
        superdesk.get_resource_service('tokens').post([token])

        # Add the access token to the Redis cache and set it to
        # expire at the appropriate time.
        app.redis.set(token['access_token'], user_id)
        app.redis.expire(token['access_token'], expires_in)
